@@ -6,9 +6,12 @@ from .memoryHistoryStorage import MemoryHistoryStorage
 from .metatraderAccountModel import MetatraderAccountModel
 from .historyStorage import HistoryStorage
 from .clients.timeoutException import TimeoutException
-from .models import random_id
+from .models import random_id, TradeOptions, MetatraderSymbolSpecification, MetatraderAccountInformation, \
+    MetatraderPosition, MetatraderOrder, MetatraderHistoryOrders, MetatraderDeals, MetatraderTradeResponse, \
+    MetatraderSymbolPrice
+from .invalidSynchronizationModeException import InvalidSynchronizationModeException
 from datetime import datetime, timedelta
-from typing import Coroutine
+from typing import Coroutine, List
 import asyncio
 
 
@@ -34,13 +37,13 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         self._lastSynchronizationId = None
         if account.synchronization_mode == 'user':
             self._terminalState = TerminalState()
-            self._historyStorage = history_storage or MemoryHistoryStorage()
+            self._historyStorage = history_storage or MemoryHistoryStorage(account.id)
             self._websocketClient.add_synchronization_listener(account.id, self)
             self._websocketClient.add_synchronization_listener(account.id, self._terminalState)
             self._websocketClient.add_synchronization_listener(account.id, self._historyStorage)
             self._websocketClient.add_reconnect_listener(self)
 
-    def get_account_information(self) -> Coroutine:
+    def get_account_information(self) -> 'Coroutine[asyncio.Future[MetatraderAccountInformation]]':
         """Returns account information (see
         https://metaapi.cloud/docs/client/websocket/api/readTradingTerminalState/readAccountInformation/).
 
@@ -49,7 +52,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         """
         return self._websocketClient.get_account_information(self._account.id)
 
-    def get_positions(self) -> Coroutine:
+    def get_positions(self) -> 'Coroutine[asyncio.Future[List[MetatraderPosition]]]':
         """Returns positions (see
         https://metaapi.cloud/docs/client/websocket/api/readTradingTerminalState/readPositions/).
 
@@ -58,7 +61,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         """
         return self._websocketClient.get_positions(self._account.id)
 
-    def get_position(self, position_id: str) -> Coroutine:
+    def get_position(self, position_id: str) -> 'Coroutine[asyncio.Future[MetatraderPosition]]':
         """Returns specific position (see
         https://metaapi.cloud/docs/client/websocket/api/readTradingTerminalState/readPosition/).
 
@@ -70,7 +73,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         """
         return self._websocketClient.get_position(self._account.id, position_id)
 
-    def get_orders(self) -> Coroutine:
+    def get_orders(self) -> 'Coroutine[asyncio.Future[List[MetatraderOrder]]]':
         """Returns open orders (see
         https://metaapi.cloud/docs/client/websocket/api/readTradingTerminalState/readOrders/).
 
@@ -79,7 +82,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         """
         return self._websocketClient.get_orders(self._account.id)
 
-    def get_order(self, order_id: str) -> Coroutine:
+    def get_order(self, order_id: str) -> 'Coroutine[asyncio.Future[MetatraderOrder]]':
         """Returns specific open order (see
         https://metaapi.cloud/docs/client/websocket/api/readTradingTerminalState/readOrder/).
 
@@ -91,7 +94,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         """
         return self._websocketClient.get_order(self._account.id, order_id)
 
-    def get_history_orders_by_ticket(self, ticket: str) -> Coroutine:
+    def get_history_orders_by_ticket(self, ticket: str) -> 'Coroutine[MetatraderHistoryOrders]':
         """Returns the history of completed orders for a specific ticket number (see
         https://metaapi.cloud/docs/client/websocket/api/retrieveHistoricalData/readHistoryOrdersByTicket/).
 
@@ -103,7 +106,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         """
         return self._websocketClient.get_history_orders_by_ticket(self._account.id, ticket)
 
-    def get_history_orders_by_position(self, position_id: str) -> Coroutine:
+    def get_history_orders_by_position(self, position_id: str) -> 'Coroutine[MetatraderHistoryOrders]':
         """Returns the history of completed orders for a specific position id (see
         https://metaapi.cloud/docs/client/websocket/api/retrieveHistoricalData/readHistoryOrdersByPosition/)
 
@@ -116,7 +119,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         return self._websocketClient.get_history_orders_by_position(self._account.id, position_id)
 
     def get_history_orders_by_time_range(self, start_time: datetime, end_time: datetime, offset: int = 0,
-                                         limit: int = 1000) -> Coroutine:
+                                         limit: int = 1000) -> 'Coroutine[MetatraderHistoryOrders]':
         """Returns the history of completed orders for a specific time range (see
         https://metaapi.cloud/docs/client/websocket/api/retrieveHistoricalData/readHistoryOrdersByTimeRange/)
 
@@ -132,7 +135,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         return self._websocketClient.get_history_orders_by_time_range(self._account.id, start_time, end_time,
                                                                       offset, limit)
 
-    def get_deals_by_ticket(self, ticket: str) -> Coroutine:
+    def get_deals_by_ticket(self, ticket: str) -> 'Coroutine[MetatraderDeals]':
         """Returns history deals with a specific ticket number (see
         https://metaapi.cloud/docs/client/websocket/api/retrieveHistoricalData/readDealsByTicket/).
 
@@ -144,7 +147,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         """
         return self._websocketClient.get_deals_by_ticket(self._account.id, ticket)
 
-    def get_deals_by_position(self, position_id) -> Coroutine:
+    def get_deals_by_position(self, position_id) -> 'Coroutine[MetatraderDeals]':
         """Returns history deals for a specific position id (see
         https://metaapi.cloud/docs/client/websocket/api/retrieveHistoricalData/readDealsByPosition/).
 
@@ -157,7 +160,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         return self._websocketClient.get_deals_by_position(self._account.id, position_id)
 
     def get_deals_by_time_range(self, start_time: datetime, end_time: datetime, offset: int = 0,
-                                limit: int = 1000) -> Coroutine:
+                                limit: int = 1000) -> 'Coroutine[MetatraderDeals]':
         """Returns history deals with for a specific time range (see
         https://metaapi.cloud/docs/client/websocket/api/retrieveHistoricalData/readDealsByTimeRange/).
 
@@ -182,7 +185,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         return self._websocketClient.remove_history(self._account.id)
 
     def create_market_buy_order(self, symbol: str, volume: float, stop_loss: float = None, take_profit: float = None,
-                                comment: str = None, client_id: str = None) -> Coroutine:
+                                options: TradeOptions = None) -> 'Coroutine[asyncio.Future[MetatraderTradeResponse]]':
         """Creates a market buy order (see https://metaapi.cloud/docs/client/websocket/api/trade/).
 
         Args:
@@ -190,13 +193,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             volume: Order volume.
             stop_loss: Optional stop loss price.
             take_profit: Optional take profit price.
-            comment: Optional order comment. The sum of the line lengths of the comment and the clientId must
-            be less than or equal to 27. For more information see https://metaapi.cloud/docs/client/clientIdUsage/
-            client_id: optional client-assigned id. The id value can be assigned when submitting a trade and will be
-            present on position, history orders and history deals related to the trade. You can use this field to bind
-            your trades to objects in your application and then track trade progress. The sum of the line lengths of
-            the comment and the clientId must be less than or equal to 27. For more information
-            see https://metaapi.cloud/docs/client/clientIdUsage/
+            options: Optional trade options.
 
         Returns:
             A coroutine resolving with trade result.
@@ -209,14 +206,11 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             trade_params['stopLoss'] = stop_loss
         if take_profit:
             trade_params['takeProfit'] = take_profit
-        if comment:
-            trade_params['comment'] = comment
-        if client_id:
-            trade_params['clientId'] = client_id
+        trade_params.update(options or {})
         return self._websocketClient.trade(self._account.id, trade_params)
 
     def create_market_sell_order(self, symbol: str, volume: float, stop_loss: float = None, take_profit: float = None,
-                                 comment: str = None, client_id: str = None) -> Coroutine:
+                                 options: TradeOptions = None) -> 'Coroutine[asyncio.Future[MetatraderTradeResponse]]':
         """Creates a market sell order (see https://metaapi.cloud/docs/client/websocket/api/trade/).
 
         Args:
@@ -224,13 +218,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             volume: Order volume.
             stop_loss: Optional stop loss price.
             take_profit: Optional take profit price.
-            comment: Optional order comment. The sum of the line lengths of the comment and the clientId must
-            be less than or equal to 27. For more information see https://metaapi.cloud/docs/client/clientIdUsage/
-            client_id: optional client-assigned id. The id value can be assigned when submitting a trade and will be
-            present on position, history orders and history deals related to the trade. You can use this field to bind
-            your trades to objects in your application and then track trade progress. The sum of the line lengths of
-            the comment and the clientId must be less than or equal to 27. For more information
-            see https://metaapi.cloud/docs/client/clientIdUsage/
+            options: Optional trade options.
 
         Returns:
             A coroutine resolving with trade result.
@@ -243,14 +231,12 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             trade_params['stopLoss'] = stop_loss
         if take_profit:
             trade_params['takeProfit'] = take_profit
-        if comment:
-            trade_params['comment'] = comment
-        if client_id:
-            trade_params['clientId'] = client_id
+        trade_params.update(options or {})
         return self._websocketClient.trade(self._account.id, trade_params)
 
     def create_limit_buy_order(self, symbol: str, volume: float, open_price: float, stop_loss: float = None,
-                               take_profit: float = None, comment: str = None, client_id: str = None) -> Coroutine:
+                               take_profit: float = None, options: TradeOptions = None) -> \
+            'Coroutine[asyncio.Future[MetatraderTradeResponse]]':
         """Creates a limit buy order (see https://metaapi.cloud/docs/client/websocket/api/trade/).
 
         Args:
@@ -259,13 +245,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             open_price: Order limit price.
             stop_loss: Optional stop loss price.
             take_profit: Optional take profit price.
-            comment: Optional order comment. The sum of the line lengths of the comment and the clientId must
-            be less than or equal to 27. For more information see https://metaapi.cloud/docs/client/clientIdUsage/
-            client_id: optional client-assigned id. The id value can be assigned when submitting a trade and will be
-            present on position, history orders and history deals related to the trade. You can use this field to bind
-            your trades to objects in your application and then track trade progress. The sum of the line lengths of
-            the comment and the clientId must be less than or equal to 27. For more information
-            see https://metaapi.cloud/docs/client/clientIdUsage/
+            options: Optional trade options.
 
         Returns:
             A coroutine resolving with trade result.
@@ -279,14 +259,12 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             trade_params['stopLoss'] = stop_loss
         if take_profit:
             trade_params['takeProfit'] = take_profit
-        if comment:
-            trade_params['comment'] = comment
-        if client_id:
-            trade_params['clientId'] = client_id
+        trade_params.update(options or {})
         return self._websocketClient.trade(self._account.id, trade_params)
 
     def create_limit_sell_order(self, symbol: str, volume: float, open_price: float, stop_loss: float = None,
-                                take_profit: float = None, comment: str = None, client_id: str = None) -> Coroutine:
+                                take_profit: float = None, options: TradeOptions = None) -> \
+            'Coroutine[asyncio.Future[MetatraderTradeResponse]]':
         """Creates a limit sell order (see https://metaapi.cloud/docs/client/websocket/api/trade/).
 
         Args:
@@ -295,13 +273,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             open_price: Order limit price.
             stop_loss: Optional stop loss price.
             take_profit: Optional take profit price.
-            comment: Optional order comment. The sum of the line lengths of the comment and the clientId must
-            be less than or equal to 27. For more information see https://metaapi.cloud/docs/client/clientIdUsage/
-            client_id: optional client-assigned id. The id value can be assigned when submitting a trade and will be
-            present on position, history orders and history deals related to the trade. You can use this field to bind
-            your trades to objects in your application and then track trade progress. The sum of the line lengths of
-            the comment and the clientId must be less than or equal to 27. For more information
-            see https://metaapi.cloud/docs/client/clientIdUsage/
+            options: Optional trade options.
 
         Returns:
             A coroutine resolving with trade result.
@@ -315,14 +287,12 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             trade_params['stopLoss'] = stop_loss
         if take_profit:
             trade_params['takeProfit'] = take_profit
-        if comment:
-            trade_params['comment'] = comment
-        if client_id:
-            trade_params['clientId'] = client_id
+        trade_params.update(options or {})
         return self._websocketClient.trade(self._account.id, trade_params)
 
     def create_stop_buy_order(self, symbol: str, volume: float, open_price: float, stop_loss: float = None,
-                              take_profit: float = None, comment: str = None, client_id: str = None) -> Coroutine:
+                              take_profit: float = None, options: TradeOptions = None) -> \
+            'Coroutine[asyncio.Future[MetatraderTradeResponse]]':
         """Creates a stop buy order (see https://metaapi.cloud/docs/client/websocket/api/trade/).
 
         Args:
@@ -331,13 +301,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             open_price: Order limit price.
             stop_loss: Optional stop loss price.
             take_profit: Optional take profit price.
-            comment: Optional order comment. The sum of the line lengths of the comment and the clientId must
-            be less than or equal to 27. For more information see https://metaapi.cloud/docs/client/clientIdUsage/
-            client_id: optional client-assigned id. The id value can be assigned when submitting a trade and will be
-            present on position, history orders and history deals related to the trade. You can use this field to bind
-            your trades to objects in your application and then track trade progress. The sum of the line lengths of
-            the comment and the clientId must be less than or equal to 27. For more information
-            see https://metaapi.cloud/docs/client/clientIdUsage/
+            options: Optional trade options.
 
         Returns:
             A coroutine resolving with trade result.
@@ -351,14 +315,12 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             trade_params['stopLoss'] = stop_loss
         if take_profit:
             trade_params['takeProfit'] = take_profit
-        if comment:
-            trade_params['comment'] = comment
-        if client_id:
-            trade_params['clientId'] = client_id
+        trade_params.update(options or {})
         return self._websocketClient.trade(self._account.id, trade_params)
 
     def create_stop_sell_order(self, symbol: str, volume: float, open_price: float, stop_loss: float = None,
-                               take_profit: float = None, comment: str = None, client_id: str = None) -> Coroutine:
+                               take_profit: float = None, options: TradeOptions = None) -> \
+            'Coroutine[asyncio.Future[MetatraderTradeResponse]]':
         """Creates a stop sell order (see https://metaapi.cloud/docs/client/websocket/api/trade/).
 
         Args:
@@ -367,13 +329,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             open_price: Order limit price.
             stop_loss: Optional stop loss price.
             take_profit: Optional take profit price.
-            comment: Optional order comment. The sum of the line lengths of the comment and the clientId must
-            be less than or equal to 27. For more information see https://metaapi.cloud/docs/client/clientIdUsage/
-            client_id: optional client-assigned id. The id value can be assigned when submitting a trade and will be
-            present on position, history orders and history deals related to the trade. You can use this field to bind
-            your trades to objects in your application and then track trade progress. The sum of the line lengths of
-            the comment and the clientId must be less than or equal to 27. For more information
-            see https://metaapi.cloud/docs/client/clientIdUsage/
+            options: Optional trade options.
 
         Returns:
             A coroutine resolving with trade result.
@@ -387,13 +343,11 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             trade_params['stopLoss'] = stop_loss
         if take_profit:
             trade_params['takeProfit'] = take_profit
-        if comment:
-            trade_params['comment'] = comment
-        if client_id:
-            trade_params['clientId'] = client_id
+        trade_params.update(options or {})
         return self._websocketClient.trade(self._account.id, trade_params)
 
-    def modify_position(self, position_id: str, stop_loss: float = None, take_profit: float = None) -> Coroutine:
+    def modify_position(self, position_id: str, stop_loss: float = None, take_profit: float = None) -> \
+            'Coroutine[asyncio.Future[MetatraderTradeResponse]]':
         """Modifies a position (see https://metaapi.cloud/docs/client/websocket/api/trade/).
 
         Args:
@@ -414,20 +368,14 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             trade_params['takeProfit'] = take_profit
         return self._websocketClient.trade(self._account.id, trade_params)
 
-    def close_position_partially(self, position_id: str, volume: float, comment: str = None, client_id: str = None) \
-            -> Coroutine:
+    def close_position_partially(self, position_id: str, volume: float, options: TradeOptions = None) -> \
+            'Coroutine[asyncio.Future[MetatraderTradeResponse]]':
         """Partially closes a position (see https://metaapi.cloud/docs/client/websocket/api/trade/).
 
         Args:
             position_id: Position id to modify.
             volume: Volume to close.
-            comment: Optional order comment. The sum of the line lengths of the comment and the clientId must be less
-            than or equal to 27. For more information see https://metaapi.cloud/docs/client/clientIdUsage/.
-            client_id: Optional client-assigned id. The id value can be assigned when submitting a trade and will be
-            present on position, history orders and history deals related to the trade. You can use this field to bind
-            your trades to objects in your application and then track trade progress. The sum of the line lengths of
-            the comment and the clientId must be less than or equal to 27. For more information
-            see https://metaapi.cloud/docs/client/clientIdUsage/
+            options: Optional trade options.
 
         Returns:
             A coroutine resolving with trade result.
@@ -436,24 +384,16 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             TradeException: On trade error.
         """
         trade_params = {'actionType': 'POSITION_PARTIAL', 'positionId': position_id, 'volume': volume}
-        if comment:
-            trade_params['comment'] = comment
-        if client_id:
-            trade_params['clientId'] = client_id
+        trade_params.update(options or {})
         return self._websocketClient.trade(self._account.id, trade_params)
 
-    def close_position(self, position_id: str, comment: str = None, client_id: str = None) -> Coroutine:
+    def close_position(self, position_id: str, options: TradeOptions = None) -> \
+            'Coroutine[asyncio.Future[MetatraderTradeResponse]]':
         """Fully closes a position (see https://metaapi.cloud/docs/client/websocket/api/trade/).
 
         Args:
             position_id: Position id to modify.
-            comment: Optional order comment. The sum of the line lengths of the comment and the clientId must be less
-            than or equal to 27. For more information see https://metaapi.cloud/docs/client/clientIdUsage/.
-            client_id: Optional client-assigned id. The id value can be assigned when submitting a trade and will be
-            present on position, history orders and history deals related to the trade. You can use this field to bind
-            your trades to objects in your application and then track trade progress. The sum of the line lengths of
-            the comment and the clientId must be less than or equal to 27. For more information
-            see https://metaapi.cloud/docs/client/clientIdUsage/
+            options: Optional trade options.
 
         Returns:
             A coroutine resolving with trade result.
@@ -462,25 +402,16 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             TradeException: On trade error.
         """
         trade_params = {'actionType': 'POSITION_CLOSE_ID', 'positionId': position_id}
-        if comment:
-            trade_params['comment'] = comment
-        if client_id:
-            trade_params['clientId'] = client_id
+        trade_params.update(options or {})
         return self._websocketClient.trade(self._account.id, trade_params)
 
-    def close_position_by_symbol(self, symbol: str, comment: str = None, client_id: str = None) -> Coroutine:
-        """Closes position by a symbol. Available on MT5 netting accounts only. (see
-        https://metaapi.cloud/docs/client/websocket/api/trade/).
+    def close_positions_by_symbol(self, symbol: str, options: TradeOptions = None) -> \
+            'Coroutine[asyncio.Future[MetatraderTradeResponse]]':
+        """Closes positions by a symbol (see https://metaapi.cloud/docs/client/websocket/api/trade/).
 
         Args:
             symbol: Symbol to trade.
-            comment: optional order comment. The sum of the line lengths of the comment and the clientId
-            must be less than or equal to 27. For more information see https://metaapi.cloud/docs/client/clientIdUsage/
-            client_id: Optional client-assigned id. The id value can be assigned when submitting a trade and will be
-            present on position, history orders and history deals related to the trade. You can use this field to bind
-            your trades to objects in your application and then track trade progress. The sum of the line lengths of
-            the comment and the clientId must be less than or equal to 27. For more information
-            see https://metaapi.cloud/docs/client/clientIdUsage/
+            options: Optional trade options.
 
         Returns:
             A coroutine resolving with trade result.
@@ -488,15 +419,13 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         Raises:
             TradeException: On trade error.
         """
-        trade_params = {'actionType': 'POSITION_CLOSE_SYMBOL', 'symbol': symbol}
-        if comment:
-            trade_params['comment'] = comment
-        if client_id:
-            trade_params['clientId'] = client_id
+        trade_params = {'actionType': 'POSITIONS_CLOSE_SYMBOL', 'symbol': symbol}
+        trade_params.update(options or {})
         return self._websocketClient.trade(self._account.id, trade_params)
 
     def modify_order(self, order_id: str, open_price: float, stop_loss: float = None,
-                     take_profit: float = None) -> Coroutine:
+                     take_profit: float = None) -> \
+            'Coroutine[asyncio.Future[MetatraderTradeResponse]]':
         """Modifies a pending order (see https://metaapi.cloud/docs/client/websocket/api/trade/).
 
         Args:
@@ -518,7 +447,8 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             trade_params['takeProfit'] = take_profit
         return self._websocketClient.trade(self._account.id, trade_params)
 
-    def cancel_order(self, order_id: str) -> Coroutine:
+    def cancel_order(self, order_id: str) -> \
+            'Coroutine[asyncio.Future[MetatraderTradeResponse]]':
         """Cancels order (see https://metaapi.cloud/docs/client/websocket/api/trade/).
 
         Args:
@@ -556,6 +486,11 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             return await self._websocketClient.synchronize(self._account.id, synchronization_id,
                                                            starting_history_order_time, starting_deal_time)
 
+    async def initialize(self) -> Coroutine:
+        """Initializes meta api connection"""
+        if self._account.synchronization_mode == 'user':
+            await self._historyStorage.load_data_from_disk()
+
     async def subscribe(self) -> Coroutine:
         """Initiates subscription to MetaTrader terminal.
 
@@ -574,9 +509,11 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         Returns:
             Promise which resolves when subscription request was processed.
         """
+        if self._account.synchronization_mode != 'user':
+            raise InvalidSynchronizationModeException(self._account)
         return self._websocketClient.subscribe_to_market_data(self._account.id, symbol)
 
-    def get_symbol_specification(self, symbol: str) -> Coroutine:
+    def get_symbol_specification(self, symbol: str) -> 'Coroutine[asyncio.Future[MetatraderSymbolSpecification]]':
         """Retrieves specification for a symbol (see
         https://metaapi.cloud/docs/client/websocket/api/retrieveMarketData/getSymbolSpecification/).
 
@@ -584,11 +521,11 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             symbol: Symbol to retrieve specification for.
 
         Returns:
-            A coroutine which resolves when specification is retrieved.
+            A coroutine which resolves when specification MetatraderSymbolSpecification is retrieved.
         """
         return self._websocketClient.get_symbol_specification(self._account.id, symbol)
 
-    def get_symbol_price(self, symbol) -> Coroutine:
+    def get_symbol_price(self, symbol) -> 'Coroutine[asyncio.Future[MetatraderSymbolPrice]]':
         """Retrieves specification for a symbol (see
         https://metaapi.cloud/docs/client/websocket/api/retrieveMarketData/getSymbolPrice/).
 
@@ -596,7 +533,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             symbol: Symbol to retrieve price for.
 
         Returns:
-            A coroutine which resolves when price is retrieved.
+            A coroutine which resolves when price MetatraderSymbolPrice is retrieved.
         """
         return self._websocketClient.get_symbol_price(self._account.id, symbol)
 
@@ -607,6 +544,8 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         Returns:
             Local copy of terminal state.
         """
+        if self._account.synchronization_mode != 'user':
+            raise InvalidSynchronizationModeException(self._account)
         return self._terminalState
 
     @property
@@ -616,6 +555,8 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         Returns:
             Local history storage.
         """
+        if self._account.synchronization_mode != 'user':
+            raise InvalidSynchronizationModeException(self._account)
         return self._historyStorage
 
     def add_synchronization_listener(self, listener):
@@ -626,6 +567,8 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         """
         if self._account.synchronization_mode == 'user':
             self._websocketClient.add_synchronization_listener(self._account.id, listener)
+        else:
+            raise InvalidSynchronizationModeException(self._account)
 
     def remove_synchronization_listener(self, listener):
         """Removes synchronization listener for specific account. Use this method for accounts in user
@@ -636,6 +579,8 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         """
         if self._account.synchronization_mode == 'user':
             self._websocketClient.remove_synchronization_listener(self._account.id, listener)
+        else:
+            raise InvalidSynchronizationModeException(self._account)
 
     async def on_connected(self):
         """Invoked when connection to MetaTrader terminal established.
@@ -656,6 +601,8 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             synchronization_id: Synchronization request id.
         """
         self._dealsSynchronized[synchronization_id] = True
+        if self._account.synchronization_mode == 'user':
+            await self._historyStorage.update_disk_storage()
 
     async def on_order_synchronization_finished(self, synchronization_id: str):
         """Invoked when a synchronization of history orders on a MetaTrader account have finished
