@@ -104,7 +104,7 @@ class AutoMockAccount(MetatraderAccount):
 account = MockAccount(MagicMock(), MagicMock(), MagicMock())
 auto_account = AutoMockAccount(MagicMock(), MagicMock(), MagicMock())
 client = MockClient('token')
-api = MetaApiConnection(client, account)
+api = None
 
 
 @pytest.fixture(autouse=True)
@@ -414,7 +414,8 @@ class TestMetaApiConnection:
             'orderId': 46870472
         }
         client.trade = AsyncMock(return_value=trade_result)
-        actual = await api.create_market_buy_order('GBPUSD', 0.07, 0.9, 2.0, 'comment', 'TE_GBPUSD_7hyINWqAlE')
+        actual = await api.create_market_buy_order('GBPUSD', 0.07, 0.9, 2.0, {'comment': 'comment',
+                                                                              'clientId': 'TE_GBPUSD_7hyINWqAlE'})
         assert actual == trade_result
         client.trade.assert_called_with('accountId', {'actionType': 'ORDER_TYPE_BUY', 'symbol': 'GBPUSD',
                                                       'volume': 0.07, 'stopLoss': 0.9, 'takeProfit': 2.0,
@@ -429,7 +430,8 @@ class TestMetaApiConnection:
             'orderId': 46870472
         }
         client.trade = AsyncMock(return_value=trade_result)
-        actual = await api.create_market_sell_order('GBPUSD', 0.07, 0.9, 2.0, 'comment', 'TE_GBPUSD_7hyINWqAlE')
+        actual = await api.create_market_sell_order('GBPUSD', 0.07, 0.9, 2.0, {'comment': 'comment',
+                                                                               'clientId': 'TE_GBPUSD_7hyINWqAlE'})
         assert actual == trade_result
         client.trade.assert_called_with('accountId', {'actionType': 'ORDER_TYPE_SELL', 'symbol': 'GBPUSD',
                                                       'volume': 0.07, 'stopLoss': 0.9, 'takeProfit': 2.0,
@@ -444,7 +446,8 @@ class TestMetaApiConnection:
             'orderId': 46870472
         }
         client.trade = AsyncMock(return_value=trade_result)
-        actual = await api.create_limit_buy_order('GBPUSD', 0.07, 1.0, 0.9, 2.0, 'comment', 'TE_GBPUSD_7hyINWqAlE')
+        actual = await api.create_limit_buy_order('GBPUSD', 0.07, 1.0, 0.9, 2.0, {'comment': 'comment',
+                                                                                  'clientId': 'TE_GBPUSD_7hyINWqAlE'})
         assert actual == trade_result
         client.trade.assert_called_with('accountId', {'actionType': 'ORDER_TYPE_BUY_LIMIT', 'symbol': 'GBPUSD',
                                                       'volume': 0.07, 'openPrice': 1.0, 'stopLoss': 0.9,
@@ -460,7 +463,8 @@ class TestMetaApiConnection:
             'orderId': 46870472
         }
         client.trade = AsyncMock(return_value=trade_result)
-        actual = await api.create_limit_sell_order('GBPUSD', 0.07, 1.0, 0.9, 2.0, 'comment', 'TE_GBPUSD_7hyINWqAlE')
+        actual = await api.create_limit_sell_order('GBPUSD', 0.07, 1.0, 0.9, 2.0, {'comment': 'comment',
+                                                                                   'clientId': 'TE_GBPUSD_7hyINWqAlE'})
         assert actual == trade_result
         client.trade.assert_called_with('accountId', {'actionType': 'ORDER_TYPE_SELL_LIMIT', 'symbol': 'GBPUSD',
                                                       'volume': 0.07, 'openPrice': 1.0, 'stopLoss': 0.9,
@@ -476,7 +480,8 @@ class TestMetaApiConnection:
             'orderId': 46870472
         }
         client.trade = AsyncMock(return_value=trade_result)
-        actual = await api.create_stop_buy_order('GBPUSD', 0.07, 1.0, 0.9, 2.0, 'comment', 'TE_GBPUSD_7hyINWqAlE')
+        actual = await api.create_stop_buy_order('GBPUSD', 0.07, 1.0, 0.9, 2.0, {'comment': 'comment',
+                                                                                 'clientId': 'TE_GBPUSD_7hyINWqAlE'})
         assert actual == trade_result
         client.trade.assert_called_with('accountId', {'actionType': 'ORDER_TYPE_BUY_STOP', 'symbol': 'GBPUSD',
                                                       'volume': 0.07, 'openPrice': 1.0, 'stopLoss': 0.9,
@@ -492,7 +497,8 @@ class TestMetaApiConnection:
             'orderId': 46870472
         }
         client.trade = AsyncMock(return_value=trade_result)
-        actual = await api.create_stop_sell_order('GBPUSD', 0.07, 1.0, 0.9, 2.0, 'comment', 'TE_GBPUSD_7hyINWqAlE')
+        actual = await api.create_stop_sell_order('GBPUSD', 0.07, 1.0, 0.9, 2.0, {'comment': 'comment',
+                                                                                  'clientId': 'TE_GBPUSD_7hyINWqAlE'})
         assert actual == trade_result
         client.trade.assert_called_with('accountId', {'actionType': 'ORDER_TYPE_SELL_STOP', 'symbol': 'GBPUSD',
                                                       'volume': 0.07, 'openPrice': 1.0, 'stopLoss': 0.9,
@@ -541,17 +547,17 @@ class TestMetaApiConnection:
         client.trade.assert_called_with('accountId', {'actionType': 'POSITION_CLOSE_ID', 'positionId': '46870472'})
 
     @pytest.mark.asyncio
-    async def test_close_position_by_symbol(self):
-        """Should close position by symbol."""
+    async def test_close_positions_by_symbol(self):
+        """Should close positions by symbol."""
         trade_result = {
             'error': 10009,
             'description': 'TRADE_RETCODE_DONE',
             'orderId': 46870472
         }
         client.trade = AsyncMock(return_value=trade_result)
-        actual = await api.close_position_by_symbol('EURUSD')
+        actual = await api.close_positions_by_symbol('EURUSD')
         assert actual == trade_result
-        client.trade.assert_called_with('accountId', {'actionType': 'POSITION_CLOSE_SYMBOL', 'symbol': 'EURUSD'})
+        client.trade.assert_called_with('accountId', {'actionType': 'POSITIONS_CLOSE_SYMBOL', 'symbol': 'EURUSD'})
 
     @pytest.mark.asyncio
     async def test_modify_order(self):
@@ -701,6 +707,7 @@ class TestMetaApiConnection:
     async def test_wait_sync_complete_user_mode(self):
         """Should wait until synchronization complete in user mode."""
         assert not (await api.is_synchronized())
+        api._historyStorage.update_disk_storage = AsyncMock()
         try:
             await api.wait_synchronized('synchronizationId', 1, 10)
             raise Exception('TimeoutError is expected')
@@ -713,6 +720,7 @@ class TestMetaApiConnection:
         await promise
         assert pytest.approx(0, 10) == (datetime.now() - start_time).seconds * 1000
         assert (await api.is_synchronized('synchronizationId'))
+        api._historyStorage.update_disk_storage.assert_called()
 
     @pytest.mark.asyncio
     async def test_time_out_waiting_for_sync_user_mode(self):
@@ -743,3 +751,10 @@ class TestMetaApiConnection:
         api.subscribe = AsyncMock()
         await api.on_reconnected()
         api.subscribe.assert_called_with()
+
+    @pytest.mark.asyncio
+    async def test_load_history_storage_from_disk(self):
+        """Should load data to history storage from disk."""
+        api._historyStorage.load_data_from_disk = AsyncMock()
+        await api.initialize()
+        api._historyStorage.load_data_from_disk.assert_called()
